@@ -2,7 +2,7 @@ import os
 import os.path
 import logging
 from flask import Flask, abort, g
-from wikked.wiki import Wiki
+from wikked.wiki import Wiki, WikiParameters
 
 
 # Create the main app.
@@ -24,6 +24,8 @@ app.config.setdefault('UPDATE_WIKI_ON_START', True)
 app.config.setdefault('WIKI_AUTO_RELOAD', False)
 app.config.setdefault('WIKI_ASYNC_UPDATE', False)
 app.config.setdefault('BROKER_URL', 'sqla+sqlite:///%(root)s/.wiki/broker.db')
+app.config.setdefault('PROFILE', False)
+app.config.setdefault('PROFILE_DIR', None)
 
 
 # Find the wiki root, and further configure the app if there's a
@@ -48,6 +50,13 @@ if app.config['DEBUG']:
     })
 
 
+# Profiling
+if app.config['PROFILE']:
+    profile_dir = app.config['PROFILE_DIR']
+    from werkzeug.contrib.profiler import ProfilerMiddleware
+    app.wsgi_app = ProfilerMiddleware(app.wsgi_app, profile_dir=profile_dir)
+
+
 # Customize logging.
 if app.config['DEBUG']:
     l = logging.getLogger('wikked')
@@ -67,6 +76,7 @@ def set_app_wiki_params(params):
 
 app.set_wiki_params = set_app_wiki_params
 app.wiki_updater = None
+app.set_wiki_params(WikiParameters(wiki_root))
 
 
 # Set the wiki as a request global, and open/close the database.
