@@ -25,13 +25,19 @@ class InitializationError(Exception):
     pass
 
 
+NORMAL_CONTEXT = 0
+INIT_CONTEXT = 1
+BACKGROUND_CONTEXT = 2
+
+
 class WikiParameters(object):
     """ An object that defines how a wiki gets initialized.
     """
-    def __init__(self, root=None):
+    def __init__(self, root=None, ctx=NORMAL_CONTEXT):
         if root is None:
             root = os.getcwd()
         self.root = root
+        self.context = ctx
         self.formatters = self.getFormatters()
         self.wiki_updater = self.getWikiUpdater()
         self._config = None
@@ -48,61 +54,15 @@ class WikiParameters(object):
         return FileSystem(self.root, self.config)
 
     def index_factory(self):
-        if self._index_factory is None:
-            index_type = self.config.get('wiki', 'indexer')
-            if index_type == 'whoosh':
-                def impl():
-                    from wikked.indexer.whooshidx import WhooshWikiIndex
-                    return WhooshWikiIndex()
-                self._index_factory = impl
-            elif index_type == 'elastic':
-                def impl():
-                    from wikked.indexer.elastic import ElasticWikiIndex
-                    return ElasticWikiIndex()
-                self._index_factory = impl
-            else:
-                raise InitializationError("No such indexer: " + index_type)
+        self._ensureIndexFactory()
         return self._index_factory()
 
     def db_factory(self):
         from wikked.db.sql import SQLDatabase
         return SQLDatabase(self.config)
 
-    def scm_factory(self, for_init=False):
-        if self._scm_factory is None:
-            try:
-                scm_type = self.config.get('wiki', 'sourcecontrol')
-            except NoOptionError:
-                # Auto-detect
-                if os.path.isdir(os.path.join(self.root, '.hg')):
-                    scm_type = 'hg'
-                elif os.path.isdir(os.path.join(self.root, '.git')):
-                    scm_type = 'git'
-                else:
-                    # Default to Mercurial. Yes. I just decided that myself.
-                    scm_type = 'hg'
-
-            if scm_type == 'hg':
-                client = None
-                if not for_init:
-                    # Only create the command server once.
-                    import hglib
-                    client = hglib.open(self.root)
-
-                def impl():
-                    from wikked.scm.mercurial import (
-                        MercurialCommandServerSourceControl)
-                    return MercurialCommandServerSourceControl(
-                        self.root, client)
-                self._scm_factory = impl
-            elif scm_type == 'git':
-                def impl():
-                    from wikked.scm.git import GitLibSourceControl
-                    return GitLibSourceControl(self.root)
-                self._scm_factory = impl
-            else:
-                raise InitializationError(
-                    "No such source control: " + scm_type)
+    def scm_factory(self):
+        self._ensureScmFactory()
         return self._scm_factory()
 
     def auth_factory(self):
@@ -152,6 +112,66 @@ class WikiParameters(object):
         config.read([config_path, local_config_path])
         return config
 
+    def _ensureIndexFactory(self):
+        if self._index_factory is None:
+            index_type = self.config.get('wiki', 'indexer')
+            if index_type == 'whoosh':
+                def impl():
+                    from wikked.indexer.whooshidx import WhooshWikiIndex
+                    return WhooshWikiIndex()
+                self._index_factory = impl
+            elif index_type == 'elastic':
+                def impl():
+                    from wikked.indexer.elastic import ElasticWikiIndex
+                    return ElasticWikiIndex()
+                self._index_factory = impl
+            else:
+                raise InitializationError("No such indexer: " + index_type)
+
+    def _ensureScmFactory(self):
+        if self._scm_factory is None:
+            try:
+                scm_type = self.config.get('wiki', 'sourcecontrol')
+            except NoOptionError:
+                # Auto-detect
+                if os.path.isdir(os.path.join(self.root, '.hg')):
+                    scm_type = 'hg'
+                elif os.path.isdir(os.path.join(self.root, '.git')):
+                    scm_type = 'git'
+                else:
+                    # Default to Mercurial. Yes. I just decided that myself.
+                    scm_type = 'hg'
+
+            if self.context != NORMAL_CONTEXT and scm_type == 'hg':
+                # Quick workaround for when we're creating a new repo,
+                # or running background tasks.
+                # We'll be using the `hg` process instead of the command
+                # server, since there's no repo there yet, or we just don't
+                # want to spawn a new process unless we want to.
+                logger.debug("Forcing `hgexe` source-control for new repo.")
+                scm_type = 'hgexe'
+
+            if scm_type == 'hg':
+                def impl():
+                    from wikked.scm.mercurial import MercurialCommandServerSourceControl
+                    return MercurialCommandServerSourceControl(self.root)
+                self._scm_factory = impl
+
+            elif scm_type == 'hgexe':
+                def impl():
+                    from wikked.scm.mercurial import MercurialSourceControl
+                    return MercurialSourceControl(self.root)
+                self._scm_factory = impl
+
+            elif scm_type == 'git':
+                def impl():
+                    from wikked.scm.git import GitLibSourceControl
+                    return GitLibSourceControl(self.root)
+                self._scm_factory = impl
+            else:
+                raise InitializationError(
+                    "No such source control: " + scm_type)
+
 
 class EndpointInfo(object):
     def __init__(self, name):
@@ -163,7 +183,7 @@ class EndpointInfo(object):
 class Wiki(object):
     """ The wiki class! This is where the magic happens.
     """
-    def __init__(self, parameters, for_init=False):
+    def __init__(self, parameters):
         """ Creates a new wiki instance. It won't be fully functional
             until you call `start`, which does the actual initialization.
             This gives you a chance to customize a few more things before
@@ -186,7 +206,7 @@ class Wiki(object):
         self.fs = parameters.fs_factory()
         self.index = parameters.index_factory()
         self.db = parameters.db_factory()
-        self.scm = parameters.scm_factory(for_init)
+        self.scm = parameters.scm_factory()
         self.auth = parameters.auth_factory()
 
         self._wiki_updater = parameters.wiki_updater
@@ -290,16 +310,17 @@ class Wiki(object):
         for url in self.db.getPageUrls(subdir):
             yield url
 
-    def getPages(self, subdir=None, meta_query=None):
+    def getPages(self, subdir=None, meta_query=None, fields=None):
         """ Gets all the pages in the wiki, or in the given sub-directory.
         """
-        for page in self.db.getPages(subdir, meta_query):
+        for page in self.db.getPages(subdir=subdir, meta_query=meta_query,
+                fields=fields):
             yield page
 
-    def getPage(self, url):
+    def getPage(self, url, fields=None):
         """ Gets the page for a given URL.
         """
-        return self.db.getPage(url)
+        return self.db.getPage(url, fields=fields)
 
     def setPage(self, url, page_fields):
         """ Updates or creates a page for a given URL.
